<?php

namespace Drupal\splide\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Xss;
use Drupal\blazy\Plugin\Field\FieldFormatter\BlazyFileFormatterBase;
use Drupal\splide\SplideDefault;

/**
 * Base class for splide image and file ER formatters.
 */
abstract class SplideFileFormatterBase extends BlazyFileFormatterBase {

  use SplideFormatterTrait;

  /**
   * {@inheritdoc}
   */
  protected static $namespace = 'splide';

  /**
   * {@inheritdoc}
   */
  protected static $itemId = 'slide';

  /**
   * {@inheritdoc}
   */
  protected static $itemPrefix = 'slide';

  /**
   * {@inheritdoc}
   */
  protected static $captionId = 'caption';

  /**
   * {@inheritdoc}
   */
  protected static $navId = 'nav';

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return SplideDefault::imageSettings() + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function buildElements(array &$build, $files, $langcode) {
    $settings = $this->formatter->toHashtag($build);
    $blazies  = $settings['blazies'];
    $_caption = $settings['nav_caption'] ?? NULL;
    $tn_style = $settings['thumbnail_style'] ?? NULL;
    $is_nav   = $blazies->is('nav') ?: $settings['nav'] ?? FALSE;
    $elements = $this->getElements($build, $files);

    foreach ($elements as $element) {
      // Build individual item.
      $build['items'][] = $element;

      // Build individual thumbnail.
      if ($is_nav) {
        $sets = $this->formatter->toHashtag($element);
        $item = $this->formatter->toHashtag($element, 'item', NULL);
        $nav  = ['#settings' => $sets];

        // Thumbnail usages: asNavFor pagers, dot, arrows, photobox thumbnails.
        $nav[static::$itemId] = $tn_style
          ? $this->formatter->getThumbnail($sets, $item)
          : [];

        if ($_caption && $item && $text = $item->{$_caption} ?? NULL) {
          $caption = ['#markup' => Xss::filterAdmin($text)];
          $nav[static::$captionId] = $caption;
        }

        $build[static::$navId]['items'][] = $nav;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getPluginScopes(): array {
    $captions = ['title' => $this->t('Title'), 'alt' => $this->t('Alt')];

    return [
      'namespace'       => 'splide',
      'nav'             => TRUE,
      'thumb_captions'  => $captions,
      'thumb_positions' => TRUE,
    ] + parent::getPluginScopes();
  }

}
